/*
 * YAFFS: Yet another FFS. A NAND-flash specific file system.
 * yaffscfg.c  The configuration for the "direct" use of yaffs.
 *
 * This file is intended to be modified to your requirements.
 * There is no need to redistribute this file.
 */

#include "tx_api.h"
#include <errno.h>
#include "gpiomux_def.h"
#include "bsp_drivers.h"
#include "bsp_fs.h"
#include "ncc_init.h"	/* for bsp_printf() */

#include "yaffscfg.h"
#include "yaffs_guts.h"
#include "yaffsfs.h"
#include "yaffs_fileem2k.h"
#include "yaffs_nandemul2k.h"
#include "yaffs_ynand.h"
#include "yaffs_api.h"
#include "yaffs_nand.h"
#include "nand_parts.h"
#include "nvram_uboot.h"

const char *yaffs_fs_c_version =
    "$Id: yaffscfg2k.c,v 1.16.2.3 2011/02/10 22:31:55 cgordon Exp $";
extern const char *yaffs_guts_c_version;

// extern int	busyWait;
// extern int	busyWait2;
extern int writeWait;
extern int addrRdWait;
extern int sprRdWait;
extern int eraseWait;

/* This is really not used, but the structure and access functions are
 * surprisingly supported */
static LIST_HEAD(yaffs_dev_list);

/* forward references and prototypes */
int yaffs_readPartitionTable(char *devName, yaffs_Device *dev);
int yaffs_FindYaffsFileSystem(unsigned int *fsStart, unsigned int *fsSize);
static char *yaffs_dump_dev(char *buf, yaffs_Device * dev);

// Configuration for:
// /ram  2MB ramdisk
// /boot 2MB boot disk (flash)
// /flash 14MB flash disk (flash)
// NB Though /boot and /flash occupy the same physical device they
// are still distinct "yaffs_Devices. You may think of these as partitions
// using non-overlapping areas in the same device.
//

#include "yaffs_ramdisk.h"
#include "yaffs_flashif.h"
#include "yaffs_nandemul2k.h"

/* This flag enables the yaffs filesystem */
#if	BSP_YAFFS_DRIVER_ENABLE
#define	NAND_DEV

#if	(BSP_NVRAM_DRIVER == BSP_NVRAM_NAND)
/* This flag turns on the boot sector in the nand flash */
#define	ROOT_DEV
#endif

#endif
/* set this flag as you see fit. It provides a RAM volume in yaffs format.
 * In order to access it through clib, an entry by the same name and media
 * type NAND must be added to fs_conf in startfilesystem.c.
 * The access functions provided in the table work on a RAM drive. */
#define	RAM2K_DEV

extern TX_SEMAPHORE  yaffsFsSemaphore;

#ifdef RAM_DEV
static yaffs_Device ramDev;
#endif
#ifdef ROOT_DEV
static yaffs_Device rootDev;
#endif
#ifdef FLASH_DEV
static yaffs_Device flashDev;
#endif
#ifdef RAM2K_DEV
static yaffs_Device ram2kDev;
#endif
#ifdef NAND_DEV
static yaffs_Device nandDev;
#endif

/* pageAddrLen is number of bytes required for highest numbered chunk
 * (chunksPerBlock * numBlocks) */

nandDevice nand_init_tbl[] = {
		/* 16 MB part: 16 KB per block, 1024 blocks */
{"Toshiba TH58100FT", TOSHIBA_MANID, 0x79, 2, 512, 16, 32, 1024},
		/* 16 MB part: 16 KB per block, 1024 blocks */
{"Samsung KM29U128T", SAMSUNG_MANID, 0x73, 2, 512, 16, 32, 1024},
		/* 32 MB part: 16 KB per block, 2048 blocks */
{"Samsung KM29U256T", SAMSUNG_MANID, 0x75, 2, 512, 16, 32, 2048},
		/* 64 MB part: 16 KB per block, 4096 blocks */
{"Samsung KM29U256T", SAMSUNG_MANID, 0x76, 3, 512, 16, 32, 4096},
		/* 128 MB part: 128 KB per block, 1024 blocks */
{"STMicro W3B2AZA6 ", STM_MANID, 0xf1, 2, 2048, 64, 64, 1024},
		/* 64 MB part: 16 KB per block, 2048 blocks */
{"STMicro NAND512W3A2D ", STM_MANID, 0x76, 3, 512, 16, 32, 4096},
{NULL, 0, 0, 0, 0, 0, 0, 0}
};

yaffsfs_DeviceConfiguration yaffsfs_config[] = {
#ifdef RAM_DEV
#if 0
	{ "/ram", &ramDev},
#else
	{ "/", &ramDev},
#endif
#endif	/* RAM_DEV */
#ifdef ROOT_DEV
	{ "/root", &rootDev},
#endif	/* ROOT_DEV */
#ifdef FLASH_DEV
#if	0
	{ "/flash/", &flashDev},
#else
	{ "/flash/flash", &flashDev},
#endif
#endif	/* FLASH_DEV */
#ifdef RAM2K_DEV
	{ "/ram2k", &ram2kDev},
#endif
#ifdef	NAND_DEV
	{ "/FLASH0", &nandDev},
#endif
	{(void *)0,(void *)0} /* Null entry to terminate list */
};

/*ad*
 * The yaffs file system driver runs under the NET+OS native file system,
 * and requires that BSP_INCLUDE_FILESYSTEM_FOR_CLIBRARY be set in bsp.h.
 * Setting the BSP_YAFFS_DRIVER_ENABLE constant to TRUE causes the native
 * file system initialization software to call into the yaffs initialization
 * software.
 *
 * This is done by enabling an entry in the fs_conf table of media_type NAND,
 * which results in a call to yaffs_StartUp from the init_volume_table
 * function.
 *
 * The yaffsfs_config table defines the yaffs volume names and provides a
 * structure for each volume that is initialized in order to use the volume.
 * The yaffs file system is typically instantiated on a nand flash device.
 * Since that device is used for booting and configuration storage in addition
 * to its use as a file system, the table provides a separate volume for the
 * portion of flash used as this root device.
 *
 * We never mount the root device
 * and because of this it is not accessed as a file system, but only as raw
 * flash pages and blocks.
 *
 * In order to mount the yaffs file system volume, an entry of type NAND is
 * added to the fs_conf table in startfilesystem.c. No entry is provided there
 * for the root volume.
 *
 * The partition layout must be configured in the U-Boot console, using
 * the flpart command. See the U-Boot documentation for details in using
 * this command.
 *
 * Typical connectcore9p9360_a partition format is as follows:
 * <list type = "ordered">
 * <item> 0x000000-0x00C0000 u-boot image store</item>
 * <item> 0x0C0000-0x0140000 nvram configuration storage</item>
 * <item> 0x140000-0x0400000 kernel boot image store</item>
 * <item> 0x400000-<end of flash> file system</item>
 * </list>
 *
 * yaffs_StartUp is called from filesystem_init when BSP_YAFFS_DRIVER_ENABLE
 * is set to TRUE. That in turn enables an entry of media_type NAND in
 * the fs_conf table, which results in the call to yaffs_StartUp from
 * init_volume_table.
 *
 * yaffs_StartUp fills in the entries in the yaffs_Device tables
 * that are defined
 * above based on enabling the local compile flags for each device.
 * Code must be supplied to fill in parameters and functions as required,
 * including appropriate functions for small block or large block NAND
 * flash.
 *
 * If NAND_DEV is enabled, for example, yaffs_StartUp will read the device
 * identification from the NAND device and initialize the appropriate
 * functions based on finding the device in the nand_init_tbl.
 * If other physical devices are required, entries must be added to the
 * nand_init_tbl. If they are standard format devices with either 512
 * byte page size or 2048 byte page size the existing functions should
 * work with no modifications necessary.
 *
 * @external
 * @category fileSystem:fsClib:yaffs
 * @since 7.2
 * @library filesys
 * @include "yaffsfs.h"
 * @platform connectcore9p9360_a
 * @overview NAND Flash System Startup
 *
 *ad*/

int yaffs_StartUp(void)
{
	// Stuff to configure YAFFS
	// Stuff to initialise anything special (eg lock semaphore).
	yaffsfs_LocalInitialisation();

	// Set up devices
#ifdef RAM_DEV
	// /ram
	memset(&ramDev,0,sizeof(ramDev));
	ramDev.nDataBytesPerChunk = 512;
	ramDev.nChunksPerBlock = 32;
	ramDev.nReservedBlocks = 2; // Set this smaller for RAM
	ramDev.startBlock = 0; // Can use block 0
	ramDev.endBlock = 127; // Last block in 2MB.
	//ramDev.useNANDECC = 1;
	ramDev.nShortOpCaches = 0;	// Disable caching on this device.
	ramDev.genericDevice = (void *) 0;	// Used to identify the device in fstat.
	ramDev.writeChunkWithTagsToNAND = yramdisk_WriteChunkWithTagsToNAND;
	ramDev.readChunkWithTagsFromNAND = yramdisk_ReadChunkWithTagsFromNAND;
	ramDev.eraseBlockInNAND = yramdisk_EraseBlockInNAND;
	ramDev.initialiseNAND = yramdisk_InitialiseNAND;
	ramDev.isLittleEndian = 0;
#endif /* RAM_DEV */

#ifdef FLASH_DEV
	// /flash
	// Set this puppy up to use
	// the file emulation space as
	// 2kpage/64chunk per block/128MB device
	memset(&flashDev,0,sizeof(flashDev));
	flashDev.nDataBytesPerChunk = 2048;
	flashDev.nChunksPerBlock = 64;
	flashDev.nReservedBlocks = 5;
	flashDev.nCheckpointReservedBlocks = 5;
	//flashDev.checkpointStartBlock = 1;
	//flashDev.checkpointEndBlock = 20;
	flashDev.startBlock = 20;
	//flashDev.endBlock = 127; // Make it smaller
	flashDev.endBlock = yflash_GetNumberOfBlocks()-1;
	flashDev.isYaffs2 = 1;
	flashDev.wideTnodesDisabled=0;
	flashDev.nShortOpCaches = 10; // Use caches
	flashDev.genericDevice = (void *) 2;	// Used to identify device in fstat.
	flashDev.writeChunkWithTagsToNAND = yflash_WriteChunkWithTagsToNAND;
	flashDev.readChunkWithTagsFromNAND = yflash_ReadChunkWithTagsFromNAND;
	flashDev.eraseBlockInNAND = yflash_EraseBlockInNAND;
	flashDev.initialiseNAND = yflash_InitialiseNAND;
	flashDev.markNANDBlockBad = yflash_MarkNANDBlockBad;
	flashDev.queryNANDBlock = yflash_QueryNANDBlock;
	flashDev.isLittleEndian = 0;
#endif	/* FLASH_DEV */

#ifdef ROOT_DEV
	// /root
	memset(&rootDev, 0, sizeof(rootDev));
	na_identify_nand(&rootDev);

	rootDev.name = "/root";
	rootDev.nReservedBlocks = 0;
	/* enable raw access for ftp and dialog to all sectors before filesystem */
	rootDev.startBlock = 0;
	rootDev.endBlock = FILESYS_PART_START /
		   	(rootDev.nChunksPerBlock * rootDev.nDataBytesPerChunk) - 1;
	rootDev.useNANDECC = 0; // use YAFFS's ECC
	rootDev.nShortOpCaches = 0; // Use caches
	rootDev.genericDevice = (void *) 1;	// Used to identify the device in fstat.

	if (rootDev.nBytesPerSpare == 16)
	{
		rootDev.isYaffs2 = 0;
		rootDev.writeChunkToNAND = ynand_WriteChunkToROOT;
		rootDev.readChunkFromNAND = ynand_ReadChunkFromROOT;
	}
	else
	{
		rootDev.isYaffs2 = 1;
		rootDev.writeChunkWithTagsToNAND = ynand_WriteChunkWithTagsToNAND;
		rootDev.readChunkWithTagsFromNAND = ynand_ReadChunkWithTagsFromNAND;
		rootDev.markNANDBlockBad = ynand_MarkNANDBlockBad;
		rootDev.queryNANDBlock = ynand_QueryNANDBlock;
	}
	rootDev.eraseBlockInNAND = ynand_EraseBlockInNAND;
	rootDev.initialiseNAND = ynand_InitialiseNAND;
	rootDev.isLittleEndian = 1;
#endif /* ROOT_DEV */

#ifdef	NAND_DEV
	// /nand
	memset(&nandDev, 0, sizeof(nandDev));
	/* read the device and fill in endBlock, pageAddrLen, nDataBytesPerChunk,
	 * nChunksPerBlock, and nBytesPerSpare based on device ID. */
	na_identify_nand(&nandDev);

	nandDev.name = "/FLASH0";
	nandDev.nReservedBlocks = 2;
	nandDev.nCheckpointReservedBlocks = 2;
	nandDev.startBlock = FILESYS_PART_START /
		   	(nandDev.nChunksPerBlock * nandDev.nDataBytesPerChunk);
	if (nandDev.nBytesPerSpare == 16)
	{
		nandDev.isYaffs2 = 0;
		nandDev.writeChunkToNAND = ynand_WriteChunkToNAND;
		nandDev.readChunkFromNAND = ynand_ReadChunkFromNAND;
	}
	else
	{
		nandDev.isYaffs2 = 1;
		nandDev.writeChunkWithTagsToNAND = ynand_WriteChunkWithTagsToNAND;
		nandDev.readChunkWithTagsFromNAND = ynand_ReadChunkWithTagsFromNAND;
		nandDev.markNANDBlockBad = ynand_MarkNANDBlockBad;
		nandDev.queryNANDBlock = ynand_QueryNANDBlock;
	}
	nandDev.wideTnodesDisabled=0;
	nandDev.nShortOpCaches = 10; // Use caches
	nandDev.genericDevice = (void *) 4;	// Used to identify device in fstat.
	nandDev.eraseBlockInNAND = ynand_EraseBlockInNAND;
	nandDev.initialiseNAND = ynand_InitialiseNAND;
	nandDev.isLittleEndian = 0;
#endif

#ifdef	RAM2K_DEV
	// /ram2k
	// Set this puppy up to use
	// the file emulation space as
	// 2kpage/64chunk per block/128MB device
	memset(&ram2kDev,0,sizeof(ram2kDev));

	ram2kDev.name = "/ram2k";
	ram2kDev.nDataBytesPerChunk = nandemul2k_GetBytesPerChunk();
	ram2kDev.nChunksPerBlock = nandemul2k_GetChunksPerBlock();
	ram2kDev.nReservedBlocks = 5;
	ram2kDev.startBlock = 0; // First block after /boot
	//ram2kDev.endBlock = 127; // Last block in 16MB
	ram2kDev.endBlock = nandemul2k_GetNumberOfBlocks() - 1;
	ram2kDev.isYaffs2 = 1;
	ram2kDev.nShortOpCaches = 10; // Use caches
	ram2kDev.genericDevice = (void *) 3;	// Used to identify device in fstat.
	ram2kDev.writeChunkWithTagsToNAND = nandemul2k_WriteChunkWithTagsToNAND;
	ram2kDev.readChunkWithTagsFromNAND = nandemul2k_ReadChunkWithTagsFromNAND;
	ram2kDev.eraseBlockInNAND = nandemul2k_EraseBlockInNAND;
	ram2kDev.initialiseNAND = nandemul2k_InitialiseNAND;
	ram2kDev.markNANDBlockBad = nandemul2k_MarkNANDBlockBad;
	ram2kDev.queryNANDBlock = nandemul2k_QueryNANDBlock;
#endif	/* RAM2K_DEV */

	yaffs_initialise(yaffsfs_config);
#ifdef	ROOT_DEV
#if	((defined BSP_UBOOT_ENV_SHARE) && (BSP_UBOOT_ENV_SHARE == TRUE))
	/*
	 * now that root dev is set up, read the partition table in the
	 * fixed critical configuration area to determine where the filesystem
	 * will start.
	 */
	if (yaffs_readPartitionTable("/root", &rootDev) != 0)
	{
		/* big problem -- we don't have environment configuration in nand */
		bsp_printf("you must configure NETOS OS partition in U-Boot\n\r");
		return -1;
	}
#endif
#ifdef	NAND_DEV
	{
		unsigned int fsStart, fsSize;
		/* update the nandDev.startBlock based on partition table entry.
		 * We get Start and size from a table such as below. We have to
	 	 * find type NVPT_FILESYSTEM and NVFS_YAFFS. */
		if (yaffs_FindYaffsFileSystem(&fsStart, &fsSize) != 0)
		{
			bsp_printf("you must configure YAFFS partition in U-Boot\n\r");
			return -1;	/* there is no file system partition */
		}

		nandDev.startBlock = fsStart /
				((nandDev.nChunksPerBlock)*(nandDev.nDataBytesPerChunk));
		/* endBlock was set in na_identify_nand based on device type */
	/*
Nr | Name         |  Start     | Size       | Type          | FS    | Flags
---------------------------------------------------------------------------
0 | U-Boot       |          0 |    768 KiB | U-Boot        | None  | fixed
1 | NVRAM        |    768 KiB |    512 KiB | NVRAM         | None  | fixed
2 | FPGA         |   1280 KiB |      1 MiB | FPGA          | None  | fixed
3 | NetOS-Kernel |   2304 KiB |   2816 KiB | NET+OS-Kernel | None  |
4 | NetOS-FS     |      5 MiB |     59 MiB | Filesystem    | YAFFS |
	*/
	}
#endif
#endif
	return 0;
}

/* This is a Digi API function. It is here because yaffsfs_config
 * is static. */

int yaffs_get_dev_stat(const char *devName, char *pBuf, int buflen)
{
	yaffsfs_DeviceConfiguration *pCfg = yaffsfs_config;
	struct yaffs_DeviceStruct *pDev = NULL;
	int rc = 1;

	while (pCfg->prefix && pCfg->dev)
	{
		if (strcmp(devName, pCfg->prefix) == 0)
		{
			rc = 0;
			pDev = pCfg->dev;
			break;
		}
		pCfg++;
	}
	if (pDev)
	{
		char *pd = pBuf;
		int totlen = 0, clen;

		pd = yaffs_dump_dev(pd + totlen, pDev);
		totlen = (unsigned int) pd - (unsigned int) pBuf;
		pd = pBuf;

		clen = snprintf(pd + totlen, buflen - totlen,
					   	"isMounted.......... %d\n\r",
				pDev->isMounted);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"isCheckpointed..... %d\n\r",
				pDev->isCheckpointed);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"nDataBytesPerChunk. %d\n\r",
				pDev->nDataBytesPerChunk);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"pageAddrLen........ %d\n\r",
				pDev->pageAddrLen);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"nChunksPerBlock.... %d\n\r",
			   	pDev->nChunksPerBlock);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"nBytesPerSpare..... %d\n\r",
				pDev->nBytesPerSpare);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"nReservedBlocks.... %d\n\r",
				pDev->nReservedBlocks);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
			   	"nCheckpointReservedBlocks %d\n\r",
			   	pDev->nCheckpointReservedBlocks);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"currentDirtyChecker %d\n\r",
				pDev->currentDirtyChecker);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"ErasureFailures.... %d\n\r",
				pDev->nErasureFailures);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"nDeletions......... %d\n\r",
				pDev->nDeletions);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"nUnmarkedDeletions. %d\n\r",
				pDev->nUnmarkedDeletions);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"allocationBlock..... %d\n\r",
				pDev->allocationBlock);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"allocationPage...... %d\n\r",
				pDev->allocationPage);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"sequenceNumber...... %d\n\r",
				pDev->sequenceNumber);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"oldestDirtySequence. %d\n\r",
				pDev->oldestDirtySequence);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
		clen = snprintf(pd + totlen, buflen - totlen,
					   	"allocationBlockFinder %d\n\r",
				pDev->allocationBlockFinder);
		if (clen < (buflen - totlen)) { totlen += clen;} else { goto outOfHere;}
	}
outOfHere:
	*(pBuf + buflen) = '\0';	/* in case we filled the buffer */
	return rc;
}

/* This is borrowed from the /proc interface in yaffs_fs.c */

static char *yaffs_dump_dev(char *buf, yaffs_Device * dev)
{
	buf += sprintf(buf, "YAFFS built:" __DATE__ " " __TIME__
	       "\n\r%s\n\r%s\n\r", yaffs_fs_c_version, yaffs_guts_c_version);
	buf += sprintf(buf, "Device \"%s\"\n\r", dev->name);
	buf += sprintf(buf, "nManufID........... %x\n\r", (int) dev->nManufID);
	buf += sprintf(buf, "nDeviceID.......... %x\n\r", (int) dev->nDeviceID);
	buf += sprintf(buf, "isLittleEndian..... %d\n\r", dev->isLittleEndian);
	buf += sprintf(buf, "startBlock......... %d\n\r", dev->startBlock);
	buf += sprintf(buf, "endBlock........... %d\n\r", dev->endBlock);
	buf += sprintf(buf, "chunkGroupBits..... %d\n\r", dev->chunkGroupBits);
	buf += sprintf(buf, "chunkGroupSize..... %d\n\r", dev->chunkGroupSize);
	buf += sprintf(buf, "nErasedBlocks...... %d\n\r", dev->nErasedBlocks);
	buf += sprintf(buf, "nBadBlocks......... %d\n\r", dev->nBadBlocks); /*jhl*/
	buf += sprintf(buf, "nTnodesCreated..... %d\n\r", dev->nTnodesCreated);
	buf += sprintf(buf, "nFreeTnodes........ %d\n\r", dev->nFreeTnodes);
	buf += sprintf(buf, "nObjectsCreated.... %d\n\r", dev->nObjectsCreated);
	buf += sprintf(buf, "nFreeObjects....... %d\n\r", dev->nFreeObjects);
	buf += sprintf(buf, "nFreeChunks........ %d\n\r", dev->nFreeChunks);
	buf += sprintf(buf, "nPageWrites........ %d\n\r", dev->nPageWrites);
	buf += sprintf(buf, "nPageReads......... %d\n\r", dev->nPageReads);
	buf += sprintf(buf, "nBlockErasures..... %d\n\r", dev->nBlockErasures);
	buf += sprintf(buf, "nGCCopies.......... %d\n\r", dev->nGCCopies);
	buf +=
	    sprintf(buf, "garbageCollections. %d\n\r", dev->garbageCollections);
	buf +=
	    sprintf(buf, "passiveGCs......... %d\n\r",
		    dev->passiveGarbageCollections);
	buf += sprintf(buf, "nRetriedWrites..... %d\n\r", dev->nRetriedWrites);
	buf += sprintf(buf, "nRetireBlocks...... %d\n\r", dev->nRetiredBlocks);
	buf += sprintf(buf, "eccFixed........... %d\n\r", dev->eccFixed);
	buf += sprintf(buf, "eccUnfixed......... %d\n\r", dev->eccUnfixed);
	buf += sprintf(buf, "tagsEccFixed....... %d\n\r", dev->tagsEccFixed);
	buf += sprintf(buf, "tagsEccUnfixed..... %d\n\r", dev->tagsEccUnfixed);
	buf += sprintf(buf, "cacheHits.......... %d\n\r", dev->cacheHits);
	buf += sprintf(buf, "nDeletedFiles...... %d\n\r", dev->nDeletedFiles);
	buf += sprintf(buf, "nUnlinkedFiles..... %d\n\r", dev->nUnlinkedFiles);
	buf +=
	    sprintf(buf, "nBackgroudDeletions %d\n\r", dev->nBackgroundDeletions);
	buf += sprintf(buf, "useNANDECC......... %d\n\r", dev->useNANDECC);
	buf += sprintf(buf, "isYaffs2........... %d\n\r", dev->isYaffs2);

	return buf;
}

#if	0
int yaffs_proc_read(char *page, char **start, off_t offset, int count,
			   	int *eof, void *data)
{
	struct list_head *item;
	char *buf = page;
	int step = offset;
	int n = 0;

	/* Get proc_file_read() to step 'offset' by one on each sucessive call.
	 * We use 'offset' (*ppos) to indicate where we are in devList.
	 * This also assumes the user has posted a read buffer large
	 * enough to hold the complete output; but that's life in /proc.
	 */

	*(int *)start = 1;

	/* Print header first */
	if (step == 0) {
		buf += sprintf(buf, "YAFFS built:" __DATE__ " " __TIME__
			       "\n%s\n%s\n\r", yaffs_fs_c_version,
			       yaffs_guts_c_version);
	}

	/* hold lock_kernel while traversing yaffs_dev_list */
	// lock_kernel();

	/* Locate and print the Nth entry.  Order N-squared but N is small. */
	list_for_each(item, &yaffs_dev_list) {
		yaffs_Device *dev = list_entry(item, yaffs_Device, devList);
		if (n < step) {
			n++;
			continue;
		}
		buf += sprintf(buf, "\nDevice %d \"%s\"\n\r", n, dev->name);
		buf = yaffs_dump_dev(buf, dev);
		break;
	}
	// unlock_kernel();

	return buf - page < count ? buf - page : count;
}
#endif

#ifndef	JHL_ENV
void SetCheckpointReservedBlocks(int n)
{
	flashDev.nCheckpointReservedBlocks = n;
}
#endif	/* not JHL_ENV */

